-- Freeciv - Copyright (C) 2007 - The Freeciv Project
--   This program is free software; you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation; either version 2, or (at your option)
--   any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.

-- Place Ruins at the location of the destroyed city.
function city_destroyed_callback(city, loser, destroyer)
  city.tile:create_base("Ruins", NIL)
  -- continue processing
  return false
end

signal.connect("city_destroyed", "city_destroyed_callback")

-- Add random labels to the map.
function place_map_labels()
  local mountains = 0
  local deep_oceans = 0
  local deserts = 0
  local glaciers = 0
  local selected_mountain = 0
  local selected_ocean = 0
  local selected_desert = 0
  local selected_glacier = 0

  -- Count the tiles that has a terrain type that may get a label.
  for place in whole_map_iterate() do
    local terr = place.terrain
    local tname = terr:rule_name()
    if tname == "Mountains" then
      mountains = mountains + 1
    elseif tname == "Deep Ocean" then
      deep_oceans = deep_oceans + 1
    elseif tname == "Desert" then
      deserts = deserts + 1
    elseif tname == "Glacier" then
      glaciers = glaciers + 1
    end
  end

  -- Decide if a label should be included and, in case it should, where.
  if random(1, 100) <= 75 then
    selected_mountain = random(1, mountains)
  end
  if random(1, 100) <= 75 then
    selected_ocean = random(1, deep_oceans)
  end
  if random(1, 100) <= 75 then
    selected_desert = random(1, deserts)
  end
  if random(1, 100) <= 75 then
    selected_glacier = random(1, glaciers)
  end

  -- Place the included labels at the location determined above.
  for place in whole_map_iterate() do
    local terr = place.terrain
    local tname = terr:rule_name()

    if tname == "Mountains" then
      selected_mountain = selected_mountain - 1
      if selected_mountain == 0 then
        place:set_label(_("Highest Peak"))
      end
    elseif tname == "Deep Ocean" then
      selected_ocean = selected_ocean - 1
      if selected_ocean == 0 then
        place:set_label(_("Deep Trench"))
      end
    elseif tname == "Desert" then
      selected_desert = selected_desert - 1
      if selected_desert == 0 then
        place:set_label(_("Scorched Spot"))
      end
    elseif tname == "Glacier" then
      selected_glacier = selected_glacier - 1
      if selected_glacier == 0 then
        place:set_label(_("Frozen Lake"))
      end
    end
  end

  return false
end

signal.connect("map_generated", "place_map_labels")





messages = "------------------------------------------------------------------------------\n"



function exist_ai_player()
  local exist = false
  for player in players_iterate() do
    if player.ai_controlled
       and player.nation:rule_name() ~= "Barbarian"
       and player.nation:rule_name() ~= "Pirate" then
      exist = true
    end
  end
  return exist
end





--  AIの思考ルーチンで交易路をそれなりに重視するように

function change_ai_trait_callback()
  for player in players_iterate() do
    if player.ai_controlled then
      edit.trait_mod(player, "Trader", 200)
    end
  end
end

signal.connect("map_generated", "change_ai_trait_callback")





-- AIの思考ルーチンでターン50までは領土拡張を重視するように
-- AIの思考ルーチンでターン50-150までは交易路を重視するように

ai_trait_Expansionist = false
ai_trait_Trader = false

function change_ai_trait_Expansionist_callback(turn, year)

  if turn == 1 then
    for player in players_iterate() do
      if player.ai_controlled then
        edit.trait_mod(player, "Expansionist", 1000)
        ai_trait_Expansionist = true
      end
    end
  end

  if turn == 50 then
    for player in players_iterate() do
      if player.ai_controlled then
        if ai_trait_Expansionist then
          edit.trait_mod(player, "Expansionist", -1000)
        end
        edit.trait_mod(player, "Trader", 500)
        ai_trait_Trader = true
      end
    end
  end

  if turn == 150 then
    for player in players_iterate() do
      if player.ai_controlled and ai_trait_Trader then
        edit.trait_mod(player, "Trader", -500)
      end
    end
  end
end

signal.connect("turn_started", "change_ai_trait_Expansionist_callback")






-- ゲーム設定→社会→「移民を有効にする」を有効にしている場合、
-- 以下のAIブーストイベントスクリプトは実行されない

function check_migration_setting_callback()
  if server.setting.get("migration") == "ENABLED" then
    notify.event(nil, nil, E.CHAT_MSG,
                _("[b]The server doesn't use ai boost event functions.[/b]"))
  end
end

signal.connect("map_generated", "check_migration_setting_callback")





messages = messages ..
"「移民を食料によって制限する」mgr_foodneeded を有効にしている場合、" ..
"GameLossフラグを所持していないすべてのAIの初期ユニット数を二倍にする" ..
"\n\n"

function dupe_ai_units_callback(turn, year)

  if turn == 0 then
    if server.setting.get("migration") == "ENABLED" then return end   -- pass
    dupe_units = server.setting.get("mgr_foodneeded")
    if dupe_units == "ENABLED" and exist_ai_player() then
      notify.event(nil, nil, E.CHAT_MSG,
                  _("[b]The server duplicates AI's units.[/b]"))

      for player in players_iterate() do
        if player.ai_controlled then
          for units in player:units_iterate() do
            if not (units.utype):has_flag("GameLoss") then
              edit.create_unit(player, units.tile, units.utype, 0,
                               units:get_homecity(), -1)
            end
          end
        end
      end
    end
  end
end

signal.connect("turn_started", "dupe_ai_units_callback")





messages = messages ..
"「移民対象都市までの最大距離」mgr_distance の数値を設定している場合、" ..
"その数だけAIに開拓者ユニットを5ターン毎に追加する" ..
"\n\n"

function adds_ai_settlers_callback(turn, year)
  local num_settlers = tonumber(server.setting.get("mgr_distance"))

  if (0 < num_settlers) and (num_settlers <= 10) then
    if (turn % 5 == 0) and (turn <= num_settlers * 5) and turn ~= 0
       and exist_ai_player() then

      if server.setting.get("migration") == "ENABLED" then return end   -- pass
      notify.event(nil, nil, E.CHAT_MSG,
                   _("[b]The server adds a settler to each AIs.[/b] (%d/%d)"),
                   turn/5, num_settlers)

      for player in players_iterate() do
        if player.ai_controlled then

          -- 首都の検索
          local palace_type = find.building_type("Palace")
          local capital_city = nil
          for target_city in player:cities_iterate() do
            if target_city:has_building(palace_type) then
              capital_city = target_city
            end
          end

          -- 開拓者を1体追加
          if capital_city then
            local settlers_type = find.unit_type("Settlers")
            edit.create_unit(player, capital_city.tile,
                             settlers_type, 0, nil, -1)
          end
        end
      end
    end
  end
end

signal.connect("turn_started", "adds_ai_settlers_callback")





messages = messages ..
"「移民が発生するターン確率」mgr_turninterval の数値が10以上の場合、" ..
"その数のターン間隔ごとにAIのそれぞれの都市に所属するキャラバンを追加する" ..
"\n\n"

function adds_ai_caravans_callback(turn, year)
  local num_caravan_turns = tonumber(server.setting.get("mgr_turninterval"))

  if (turn % num_caravan_turns == 0) and (num_caravan_turns >= 10)
     and turn ~= 0  and exist_ai_player() then

    if server.setting.get("migration") == "ENABLED" then return end   -- pass
    local caravan_type = find.unit_type("Caravan")
    num_caravan = num_caravan + 1
    notify.event(nil, nil, E.CHAT_MSG,
                 _("[b]The server adds a caravan to each AI cities.[/b] (%d)"),
                 num_caravan)

    for player in players_iterate() do
      if player.ai_controlled then

        -- キャラバンの追加
        for city in player:cities_iterate() do
          edit.create_unit(player, city.tile, caravan_type, 0, city, -1)
        end
      end
    end
  end
end

num_caravan = 0
signal.connect("turn_started", "adds_ai_caravans_callback")






messages = messages ..
"「自国内の都市に移民する確率」mgr_nationchance の数値が1以上の場合、" ..
"その数のターンになったらそれぞれのAIに開拓者を10体追加する" .. "\n" ..
"そして「他国の都市に移民する確率」mgr_worldchance の数だけ科学を与える" ..
"\n\n"

function adds_ai_power_callback(turn, year)
  local num_turns = tonumber(server.setting.get("mgr_nationchance"))
  local num_techs = tonumber(server.setting.get("mgr_worldchance"))
  local num_settlers = 10

  if (0 < num_turns) and (num_turns <= 100) and turn == num_turns
     and exist_ai_player() then

    if server.setting.get("migration") == "ENABLED" then return end   -- pass
    notify.event(nil, nil, E.CHAT_MSG,
                 _("[b]The server adds %d settlers to each AIs.[/b]"),
                 num_settlers)
    if (0 < num_techs) and (num_techs <= 100) then
      notify.event(nil, nil, E.CHAT_MSG,
                   _("[b]The server adds %d techs to each AIs.[/b]"),
                   num_techs)
    end

    for player in players_iterate() do
      if player.ai_controlled then

        -- 首都の検索
        local palace_type = find.building_type("Palace")
        local capital_city = nil
        for target_city in player:cities_iterate() do
          if target_city:has_building(palace_type) then
            capital_city = target_city
          end
        end

        -- 開拓者を追加
        if capital_city then
          local settlers_type = find.unit_type("Settlers")
          for i = 0 , num_settlers, 1 do
            edit.create_unit(player, capital_city.tile,
                             settlers_type, 0, nil, -1)
          end
        else
          notify.event(nil, nil, E.CHAT_MSG,
                       _("[b]The server cannot add settlers to %s " ..
                         "because there are no capital city.[/b]"),
                       (player.nation):name_translation())
        end

        -- 科学を与える
        if (0 < num_techs) and (num_techs <= 100) then
          for i = 0 , num_techs, 1 do
            edit.give_technology(player, nil, "hut")
          end
        end

      end
    end
  end
end

signal.connect("turn_started", "adds_ai_power_callback")



notify.event(nil, nil, E.CHAT_MSG, messages ..
"------------------------------------------------------------------------------\n")

